class ValidationFailedException(Exception):
    pass


class UserExistsException(Exception):
    pass


class UserNotFoundException(Exception):
    pass


class InvalidPasswordException(Exception):
    pass


class LinkExpiredException(Exception):
    pass


class IncorrectEmailException(Exception):
    pass



class Magic:
    def __init__(self, db, mlid: str, email: str, name: str):
        self.db = db
        self.mlid = mlid
        self.email = email
        self.name = name

    def get_name(self):
        return self.name

    def get_email(self):
        return self.email



import webbrowser
from typing import List
import base64

import requests


class GitHub:
    def __init__(self, client_id: str, client_secret: str, redirect_uri: str, scopes: List[str] = None):
        if scopes is None:
            scopes = []
        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect = redirect_uri
        self.scopes = scopes
        self.auth_url = f"https://github.com/login/oauth/authorize?client_id={self.client_id}&redirect_uri={self.redirect}&scope={' '.join(self.scopes)}" if scopes != [] else f"https://github.com/login/oauth/authorize?client_id={self.client_id}&redirect_uri={self.redirect}"
        self.token_url = f"https://github.com/login/oauth/access_token?client_id={self.client_id}&client_secret={self.client_secret}"

    def open_window(self):
        webbrowser.open(self.auth_url)

    def get_url(self):
        return self.auth_url

    def get_token(self, code: str):
        return requests.post(self.token_url + f"&code={code}", headers={"Accept": "application/json"}).json()[
            "access_token"
        ]


class Discord:
    def __init__(self, client_id: str, client_secret: str, redirect_uri: str, scopes: List[str]):
        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect_uri = redirect_uri
        self.scopes = scopes
        self.auth_url = f"https://discord.com/api/oauth2/authorize?response_type=code&client_id={client_id}&scope={' '.join(self.scopes)}&redirect_uri={self.redirect_uri}&prompt=consent "
        self.token_url = f"https://discord.com/api/oauth2/token"

    def open_window(self):
        webbrowser.open(self.auth_url)

    def get_url(self):
        return self.auth_url

    def get_token(self, code: str):
        payload = {
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "grant_type": "authorization_code",
            "code": code,
            "redirect_uri": self.redirect_uri
        }
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        return requests.post(self.token_url, data=payload, headers=headers).json()["access_token"]


class Spotify:
    def __init__(self, client_id: str, client_secret: str, redirect_uri: str, scopes=None):
        if scopes is None:
            scopes = []

        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect_uri = redirect_uri
        self.scopes = scopes
        self.auth_url = f"https://accounts.spotify.com/authorize?response_type=code&client_id={self.client_id}&scope={' '.join(self.scopes)}&redirect_uri={self.redirect_uri}"

    def open_window(self):
        webbrowser.open(self.auth_url)

    def get_url(self):
        return self.auth_url

    def get_token(self, code: str):
        payload = {
            "code": code,
            "redirect_uri": self.redirect_uri,
            "grant_type": "authorization_code"
        }
        headers = {
            "Content-Type": "application/x-www-form-urlencoded",
            "Authorization": "Basic "+base64.b64encode(f"{self.client_id}:{self.client_secret}".encode()).decode()
        }
        return requests.post(f"https://accounts.spotify.com/api/token", data=payload, headers=headers)["access_token"]



import bcrypt
import deta

from ..errors import ValidationFailedException, UserExistsException, UserNotFoundException, InvalidPasswordException
from ..models.user import UserEmail, User
from ..validation import validate_username, validate_email, validate_password


class OAuth:
    def __init__(self, deta_key: str, db_name: str, email_as_username: bool = False, validation: bool = True):
        deta_ = deta.Deta(deta_key)
        self.db = deta_.Base(db_name)
        self.validation = validation

        self.sign_up = self.sign_up if not email_as_username else self.sign_up_email
        self.sign_in = self.sign_in if not email_as_username else self.sign_in_email

    def sign_up(self, username: str, email: str, password: str) -> User:
        if self.db.get(username) is not None:
            raise UserExistsException(f"{username} already exists")

        if self.validation:
            if False in [validate_username(username), validate_email(email), validate_password(password)]:
                raise ValidationFailedException(f"validation failed for any one of the fields")

        hpass = bcrypt.hashpw(password.encode(), bcrypt.gensalt())
        self.db.put({"email": email, "password": hpass.decode()}, username)
        return User(self.db, email, username, hpass.decode())

    def sign_up_email(self, email: str, password: str) -> UserEmail:
        if self.db.get(email) is not None:
            raise UserExistsException(f"{email} already exists")

        if self.validation:
            if False in [validate_email(email), validate_password(password)]:
                raise ValidationFailedException("validation failed for any one of the fields")

        hpass = bcrypt.hashpw(password.encode(), bcrypt.gensalt())
        self.db.put({"password": hpass}, email)
        return UserEmail(self.db, email, hpass.decode())

    def sign_in(self, username: str, password: str) -> User:
        user = self.db.get(username)
        if user is None:
            raise UserNotFoundException(f"{username} does not exist")

        if bcrypt.checkpw(password.encode(), user["password"].encode()):
            return User(self.db, user["email"], user["key"], user["password"])
        raise InvalidPasswordException()

    def sign_in_email(self, email: str, password: str) -> UserEmail:
        user = self.db.get(email)
        if user is None:
            raise UserNotFoundException(f"{email} does not exist")

        if bcrypt.checkpw(password.encode(), user["password"].encode()):
            return UserEmail(self.db, user["key"], user["password"])
        raise InvalidPasswordException()



import re

EMAIL = re.compile(r"^\w+([-+.\']\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$")
USERNAME = re.compile(r"^[a-zA-Z0-9_\-\.]{4,}$")
PASSWORD = re.compile(
    r"(?=(.*[0-9]))(?=.*[\\!@#$%^&*()\[\]{}\-_+=~`|:;\"\'<>,.\/?])(?=.*[a-z])(?=(.*[A-Z]))(?=(.*)).{8,}"
)


def validate_username(username: str) -> bool:
    if USERNAME.match(username) is not None:
        return True
    return False


def validate_email(email: str) -> bool:
    if EMAIL.match(email) is not None:
        return True
    return False


def validate_password(password: str) -> bool:
    if PASSWORD.match(password) is not None:
        return True
    return False



import bcrypt
from deta import Deta

from ..errors import ValidationFailedException, UserExistsException
from ..validation import validate_username, validate_email, validate_password


class User:
    def __init__(self, db: Deta.Base, email: str, username: str, password: str):
        self.email = email
        self.username = username
        self.password = password
        self.db = db

    def update_username(self, new_username: str) -> None:
        if validate_username(new_username) is False:
            raise ValidationFailedException("validation of username failed")

        self.db.put({"email": self.email, "password": self.password}, new_username)
        self.db.delete(self.username)
        self.username = new_username

    def update_email(self, new_email: str) -> None:
        if validate_email(new_email) is False:
            raise ValidationFailedException("validation of email failed")

        self.db.put({"email": new_email, "password": self.password}, self.username)
        self.email = new_email

    def update_password(self, new_password: str) -> None:
        if validate_password(new_password) is False:
            raise ValidationFailedException("validation of password failed")

        hpass = bcrypt.hashpw(new_password.encode(), bcrypt.gensalt()).decode()
        self.db.put(
            {
                "email": self.email,
                "password": hpass
            },
            self.username
        )
        self.password = hpass


class UserEmail:
    def __init__(self, db: Deta.Base, email: str, password: str):
        self.email = email
        self.password = password
        self.db = db

    def update_email(self, new_email: str) -> None:
        if self.db.get(new_email) is not None:
            raise UserExistsException()

        if validate_email(new_email) is False:
            raise ValidationFailedException("validation of email failed")

        self.db.put({"password": self.password}, new_email)
        self.db.delete(self.email)
        self.email = new_email

    def update_password(self, new_password: str) -> None:
        if validate_password(new_password) is False:
            raise ValidationFailedException("validation of password failed")

        hpass = bcrypt.hashpw(new_password.encode(), bcrypt.gensalt()).decode()
        self.db.put(
            {
                "password": hpass
            },
            self.email
        )
        self.password = hpass



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="openauth",
    version="1.0.0",
    author="Py Bash",
    description="OpenAuth",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pybash1/openbase",
    project_urls={
        "Bug Tracker": "https://github.com/pybash1/openbase/issues"
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Topic :: Database",

    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6"
)



import smtplib
import uuid
import warnings

import deta

from ..errors import LinkExpiredException, IncorrectEmailException
from ..models.magic import Magic


class NotDefaultPortWarning(Warning):
    pass


BASE = """From: {} <{}>
To: {} <{}>
Content-Type: text/html
Subject:{}
<h1>{}</h1>
<a href="{}">Click here</a> to sign in to <strong>{}</strong>
"""


class MagicLink:
    def __init__(
            self,
            deta_key: str,
            db_name: str,
            host: str,
            port: int,
            email: str,
            password: str,
            org: str,
            domain: str,
            expiry: int = 900,
            name: str = ""
    ):
        if port not in [110, 995, 143, 993, 25, 26, 465, 587]:
            warnings.warn(f"port '{port}' is not one of the default ports used by mail servers")

        self.deta_ = deta.Deta(deta_key)
        self.db = self.deta_.Base(db_name)
        self.host = host
        self.port = port
        self.email = email
        self.password = password
        self.name = str(name)
        self.org = org
        self.domain = domain
        self.expiry = expiry
        self.smtp = smtplib.SMTP(self.host, self.port)

    def send_email(self, email: str, name: str = "") -> None:
        mlid = str(uuid.uuid4())
        self.db.put({"email": email, "name": name}, mlid, expire_in=self.expiry)
        email_msg = BASE.format(
            self.name,
            self.email,
            name,
            email,
            f"Link to Sign In to {self.org}",
            self.org,
            self.domain + mlid + "?email=" + email + "?name=" + name,
            self.org
        )
        self.smtp.send(email_msg)

    def validate_link(self, mlid: str, email: str, name: str) -> Magic:
        ml = self.db.get(mlid)
        if ml is None:
            raise LinkExpiredException("link has either expired or doesnt exist")

        if ml["email"] != email:
            raise IncorrectEmailException("email for link and given email does not match")

        return Magic(self.db, mlid, email, name)
